/*
	SPDX-FileCopyrightText: 2015 Graeme Gott <graeme@gottcode.org>

	SPDX-License-Identifier: GPL-3.0-or-later
*/

#ifndef KAPOW_PATHS_H
#define KAPOW_PATHS_H

class QString;

class Paths
{
public:
	static QString dataPath();
	static QString oldDataPath();
};

#endif // KAPOW_PATHS_H
